from .alert import Alert

class ICONS:
    ALERT = Alert
    CLOSE = ':/svg_cache/svg_cache/material-symbols_close-rounded.svg'
    ERROR = ':/svg_cache/svg_cache/si_error-fill.svg'
    INFO = ':/svg_cache/svg_cache/material-symbols_info-rounded.svg'
    SUCCESS = ':/svg_cache/svg_cache/ep_success-filled.svg'
    WARNING = ':/svg_cache/svg_cache/fluent_warning-28-filled.svg'
    WARNING = ':/svg_cache/svg_cache/fluent_warning-28-filled.svg'

    ARROW_UP = ':/svg_cache/svg_cache/ri_arrow-up-s-line.svg'
    ARROW_DOWN = ':/svg_cache/svg_cache/ri_arrow-down-s-line.svg'
    ARROW_LEFT = ':/svg_cache/svg_cache/ri_arrow-left-s-line.svg'
    ARROW_RIGHT = ':/svg_cache/svg_cache/ri_arrow-right-s-line.svg'

    ARROW_UP_DOUBLE = ':/svg_cache/svg_cache/ri_arrow-up-double-fill.svg'
    ARROW_DOWN_DOUBLE = ':/svg_cache/svg_cache/ri_arrow-down-double-fill.svg'
    ARROW_LEFT_DOUBLE = ':/svg_cache/svg_cache/ri_arrow-left-double-fill.svg'
    ARROW_RIGHT_DOUBLE = ':/svg_cache/svg_cache/ri_arrow-right-double-fill.svg'

    ARROW_DROP_UP = ':/svg_cache/svg_cache/ri_arrow-drop-up-fill.svg'
    ARROW_DROP_DOWN = ':/svg_cache/svg_cache/ri_arrow-drop-down-fill.svg'
    ARROW_DROP_LEFT = ':/svg_cache/svg_cache/ri_arrow-drop-left-fill.svg'
    ARROW_DROP_RIGHT = ':/svg_cache/svg_cache/ri_arrow-drop-right-fill.svg'

    CALENDAR = ':/svg_cache/svg_cache/cuida_calendar-outline.svg'

    ONLINE = ':/svg_cache/svg_cache/fluent_presence-busy-10-filled.svg'
    OFFLINE = ':/svg_cache/svg_cache/fluent_presence-busy-10-filled.svg'
    BUSY = ':/svg_cache/svg_cache/fluent_presence-busy-10-filled.svg'
    ALWAY = ':/svg_cache/svg_cache/iconamoon_clock-fill.svg'

    CHECKBOX = ':/svg_cache/svg_cache/mingcute_checkbox-fill.svg'
    STAR = ':/svg_cache/svg_cache/material-symbols_star-rounded.svg'
    DONE = ':/svg_cache/svg_cache/material-symbols_done-rounded.svg'

    # editor
    FILE_OPEN = ':/svg_cache/svg_cache/material-symbols_file-open-outline-rounded.svg'
    FILE_SAVE = ':/svg_cache/svg_cache/material-symbols_save-outline-rounded.svg'
    FILE_SAVE_AS = ':/svg_cache/svg_cache/material-symbols_save-as-outline-rounded.svg'
    PRINT = ':/svg_cache/svg_cache/material-symbols_print-outline-rounded.svg'
    UNDO = ':/svg_cache/svg_cache/material-symbols_undo-rounded.svg'
    REDO = ':/svg_cache/svg_cache/material-symbols_redo-rounded.svg'
    CONTENT_CUT = ':/svg_cache/svg_cache/material-symbols_content-cut-rounded.svg'
    CONTENT_COPY = ':/svg_cache/svg_cache/solar_copy-outline.svg'
    CONTENT_PASTE = ':/svg_cache/svg_cache/material-symbols_content-paste-rounded.svg'
    SELECT_ALL = ':/svg_cache/svg_cache/material-symbols_select-all-rounded.svg'
    WRAP_TEXT = ':/svg_cache/svg_cache/material-symbols_wrap-text-rounded.svg'
    FORMAT_BOLD = ':/svg_cache/svg_cache/material-symbols_format-bold-rounded.svg'
    FORMAT_ITALIC = ':/svg_cache/svg_cache/material-symbols_format-italic-rounded.svg'
    FORMAT_UNDERLINED = ':/svg_cache/svg_cache/material-symbols_format-underlined-rounded.svg'
    FORMAT_ALIGN_LEFT = ':/svg_cache/svg_cache/material-symbols_format-align-left-rounded.svg'
    FORMAT_ALIGN_CENTER = ':/svg_cache/svg_cache/material-symbols_format-align-center-rounded.svg'
    FORMAT_ALIGN_RIGHT = ':/svg_cache/svg_cache/material-symbols_format-align-right-rounded.svg'
    FORMAT_ALIGN_JUSTIFY = ':/svg_cache/svg_cache/material-symbols_format-align-justify-rounded.svg'

    UPLOAD = ':/svg_cache/svg_cache/upload.svg'
    CLOUD_UPLOAD_BOLD = ':/svg_cache/svg_cache/solar_cloud-upload-bold.svg'
    UPLOAD_FILE = ':/svg_cache/svg_cache/material-symbols_upload-file.svg'

    # action toolbar
    TRASH = ':/svg_cache/svg_cache/bxs_trash.svg'