width = "width"
# margin
mx = "mx"
my = "my"
mb = "mb"
mt = "mt"

# padding
p = "padding"
padding = "padding"
px = "px"
py = "py"

# screen
xs = "xs"
sm = "sm"
md = "md"
lg = "lg"
xl = "xl"
xxl = "xxl"

# css props
columns = "columns"
bgcolor = "background-color"


boxShadow = "box-shadow"

# InputProps
startAdornment = "startAdornment"
endAdornment = "endAdornment"
shrink = "shrink"

disablePadding = "disablePadding"

typography = "typography"
textTransform = "text-transform"

textAlign = "text-align"
fontStyle = "font-style"
color = "color"

border = "border" # border" # ${value}px solid
borderBottom = "border-bottom" # borderBottom" # ${value}px solid
borderBottomLeft = "border-bottom-left" # borderBottom" # ${value}px solid
borderBottomLeftRadius = "border-bottom-left-radius" # borderBottom" # ${value}px solid
borderBottomLeftColor = "border-bottom-left-color" # borderBottom" # ${value}px solid
borderBottomLeftStyle = "border-bottom-left-style" # borderBottom" # ${value}px solid
borderBottomRight = "border-bottom-right" # borderBottom" # ${value}px solid
borderBottomRightRadius = "border-bottom-right-radius" # borderBottom" # ${value}px solid
borderBottomRightColor = "border-bottom-right-color" # borderBottom" # ${value}px solid
borderBottomRightStyle = "border-bottom-right-style" # borderBottom" # ${value}px solid
borderTopLeft = "border-top-left" # borderBottom" # ${value}px solid
borderTopLeftRadius = "border-top-left-radius" # borderBottom" # ${value}px solid
borderTopLeftColor = "border-top-left-color" # borderBottom" # ${value}px solid
borderTopLeftStyle = "border-top-left-style" # borderBottom" # ${value}px solid
borderTopRight = "border-top-right" # borderBottom" # ${value}px solid
borderTopRightRadius = "border-top-right-radius" # borderBottom" # ${value}px solid
borderTopRightColor = "border-top-right-color" # borderBottom" # ${value}px solid
borderTopRightStyle = "border-top-right-style" # borderBottom" # ${value}px solid
borderColor = "border-color" # borderColor" # theme.palette[value]
borderLeft = "border-left" # borderLeft" # ${value}px solid
borderRadius = "border-radius" # borderRadius" # theme.shape.borderRadius * value
borderRight = "border-right" # borderRight" # ${value}px solid
borderTop = "border-top" # borderTop" # ${value}px solid
borderStyle = "border-style"

boxShadow = "box-shadow" # boxShadow" # theme.shadows[value]
displayPrint = "display" # displayPrint" # none
display = "display" # displayRaw" # none
alignContent = "align-content" # alignContent" # none
alignItems = "align-items" # alignItems" # none
alignSelf = "align-self" # alignSelf" # none
flex = "flex" # flex" # none
flexDirection = "flex-direction" # flexDirection" # none
flexGrow = "flex-grow" # flexGrow" # none
flexShrink = "flex-shrink" # flexShrink" # none
flexWrap = "flex-wrap" # flexWrap" # none
justifyContent = "justify-content" # justifyContent" # none
order = "order" # order" # none
gap = "gap" # gap" # theme.spacing(value)
columnSpacing = "column-gap" # columnSpacing" # theme.spacing(value)
rowSpacing = "row-gap" # rowSpacing" # theme.spacing(value)
gridColumn = "grid-column" # gridColumn" # none
gridRow = "grid-row" # gridRow" # none
gridAutoFlow = "grid-auto-flow" # gridAutoFlow" # none
gridAutoColumns = "grid-auto-columns" # gridAutoColumns" # none
gridAutoRows = "grid-auto-rows" # gridAutoRows" # none
gridTemplateColumns = "grid-template-columns" # gridTemplateColumns" # none
gridTemplateRows = "grid-template-rows" # gridTemplateRows" # none
gridTemplateAreas = "grid-template-areas" # gridTemplateAreas" # none
gridArea = "grid-area" # gridArea" # none
bgcolor = "background-color" # bgcolor" # theme.palette[value]
backgroundColor = "background-color" # bgcolor" # theme.palette[value]
color = "color" # color" # theme.palette[value]
bottom = "bottom" # bottom" # none
left = "left" # left" # none
position = "position" # position" # none
right = "right" # right" # none
top = "top" # top" # none
zIndex = "z-index" # zIndex" # theme.zIndex[value]
height = "height" # height" # none
maxHeight = "max-height" # maxHeight" # none
maxWidth = "max-width" # maxWidth" # none
minHeight = "min-height" # minHeight" # none
minWidth = "min-width" # minWidth" # none
width = "width" # width" # none
boxSizing = "box-sizing" # boxSizing" # none
m = "margin" # margin" # spacing" # theme.spacing(value)
mb = "margin-bottom" # margin-bottom" # spacing" # theme.spacing(value)
ml = "margin-left" # margin-left" # spacing" # theme.spacing(value)
mr = "margin-right" # margin-right" # spacing" # theme.spacing(value)
mt = "margin-top" # margin-top" # spacing" # theme.spacing(value)
mx = "mx" # margin-left, margin-right" # spacing" # theme.spacing(value)
my = "my" # margin-top, margin-bottom" # spacing" # theme.spacing(value)
marginInline = "margin-inline" # spacing" # theme.spacing(value)
marginInlineStart = "margin-inline-start" # spacing" # theme.spacing(value)
marginInlineEnd = "margin-inline-end" # spacing" # theme.spacing(value)
marginBlock = "margin-block" # spacing" # theme.spacing(value)
marginBlockStart = "margin-block-start" # spacing" # theme.spacing(value)
marginBlockEnd = "margin-block-end" # spacing" # theme.spacing(value)
p = "padding" # padding" # spacing" # theme.spacing(value)
pb = "padding-bottom" # padding-bottom" # spacing" # theme.spacing(value)
pl = "padding-left" # padding-left" # spacing" # theme.spacing(value)
pr = "padding-right" # padding-right" # spacing" # theme.spacing(value)
pt = "padding-top" # padding-top" # spacing" # theme.spacing(value)
px = "px" # padding-left, padding-right" # spacing" # theme.spacing(value)
py = "py" # padding-top, padding-bottom" # spacing" # theme.spacing(value)
paddingInline = "padding-inline" # spacing" # theme.spacing(value)
paddingInlineStart = "padding-inline-start" # spacing" # theme.spacing(value)
paddingInlineEnd = "padding-inline-end" # spacing" # theme.spacing(value)
paddingBloc = "padding-block" # spacing" # theme.spacing(value)
paddingBlockStart = "padding-block-start" # spacing" # theme.spacing(value)
paddingBlockEnd = "padding-block-end" # spacing" # theme.spacing(value)
typography = "font-family, font-weight, font-size, line-height, letter-spacing, text-transform" # typography" # theme.typography[value]
fontFamily = "font-family" # fontFamily" # theme.typography[value]
fontSize = "font-size" # fontSize" # theme.typography[value]
fontStyle = "font-style" # fontStyle" # theme.typography[value]
fontWeight = "font-weight" # fontWeight" # theme.typography[value]
letterSpacing = "letter-spacing" # letterSpacing" # theme.typography[value]
lineHeight = "line-height" # lineHeight" # theme.typography[value]
textAlign = "text-align" # textAlign