import os
import sys
# os.environ['QT_API'] = 'pyqt5'
from PySide6.QtWidgets import QApplication, QVBoxLayout, QWidget
# from pyqtlet2 import L, MapWidget

class MapWindow(QWidget):
    def __init__(self):
        # Setting up the widgets and layout
        super().__init__()
        # self.mapWidget = MapWidget()
        # self.layout = QVBoxLayout()
        # self.layout.addWidget(self.mapWidget)
        # self.setLayout(self.layout)

        # # Working with the maps with pyqtlet
        # self.map = L.map(self.mapWidget)
        # self.map.setView([12.97, 77.59], 10)
        # L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png').addTo(self.map)
        # self.marker = L.marker([12.934056, 77.610029])
        # self.marker.bindPopup('Maps are a treasure.')
        # self.map.addLayer(self.marker)
        # self.show()