class Alert:
    CLOSE = ':/icons/svg_cache/material-symbols_close-rounded.svg'
    ERROR = ':/icons/svg_cache/si_error-fill.svg'
    INFO = ':/icons/svg_cache/material-symbols_info-rounded.svg'
    SUCCESS = ':/icons/svg_cache/ep_success-filled.svg'
    WARNING = ':/icons/svg_cache/fluent_warning-28-filled.svg'

