from .slider import Slider
from .sliders import (
    RangeSlider,
    MultiHandleRangeSlider,
    
    DoubleSlider,
    DoubleRangeSlider,

    LabeledSlider,
    LabeledRangeSlider,
    LabeledDoubleSlider,
    LabeledDoubleRangeSlider,
)
from .demo_widget import DemoWidget