class QssProperties:
    border = "border" # borders
    borderTop = "border-top" # borders
    borderLeft = "border-left" # borders
    borderRight = "border-right" # borders
    borderBottom = "border-bottom" # borders
    borderColor = "border-color" # palette
    borderTopColor = "border-top-color" # palette
    borderRightColor = "border-right-color" # palette
    borderBottomColor = "border-bottom-color" # palette
    borderLeftColor = "border-left-color" # palette
    borderRadius = "border-radius" # shape

    # system/spacing props
    margin = "margin" # spacing" # theme.spacing(value)
    m = "margin" # spacing" # theme.spacing(value)
    mt = "margin-top" # spacing" # theme.spacing(value)
    mr = "margin-right" # spacing" # theme.spacing(value)
    mb = "margin-bottom" # spacing" # theme.spacing(value)
    ml = "margin-left" # spacing" # theme.spacing(value)
    mx = ["margin-left", "margin-right"] # spacing" # theme.spacing(value)
    my = ["margin-top", "margin-bottom"] # spacing" # theme.spacing(value)
    marginTop = "margin-top" # spacing" # theme.spacing(value)
    marginRight = "margin-right" # spacing" # theme.spacing(value)
    marginBottom = "margin-bottom" # spacing" # theme.spacing(value)
    marginLeft = "margin-left" # spacing" # theme.spacing(value)
    marginX = ["margin-left", "margin-right"] # spacing" # theme.spacing(value)
    marginY = ["margin-top", "margin-bottom"] # spacing" # theme.spacing(value
    
    padding = "padding" # spacing" # theme.spacing(value)
    p = "padding" # spacing" # theme.spacing(value)
    pt = "padding-top" # spacing" # theme.spacing(value)
    pr = "padding-right" # spacing" # theme.spacing(value)
    pb = "padding-bottom" # spacing" # theme.spacing(value)
    pl = "padding-left" # spacing" # theme.spacing(value)
    px = ["padding-left", "padding-right"] # spacing" # theme.spacing(value)
    py = ["padding-top", "padding-bottom"] # spacing" # theme.spacing(value)
    paddingTop = "padding-top" # spacing" # theme.spacing(value)
    paddingRight = "padding-right" # spacing" # theme.spacing(value)
    paddingBottom = "padding-bottom" # spacing" # theme.spacing(value)
    paddingLeft = "padding-left" # spacing" # theme.spacing(value)
    paddingX = ["padding-left", "padding-right"] # spacing" # theme.spacing(value)
    paddingY = ["padding-top", "padding-bottom"] # spacing" # theme.spacing(value

    # system/typography props
    typography = "font-family, font-weight, font-size, line-height, letter-spacing, text-transform" # theme.typography[value]
    fontFamily = "font-family" # theme.typography[value]
    fontSize = "font-size" # theme.typography[value]
    fontStyle = "font-style" # theme.typography[value]
    fontWeight = "font-weight" # theme.typography[value]
    letterSpacing = "letter-spacing" # theme.typography[value]
    lineHeight = "line-height" # theme.typography[value]
    textAlign = "text-align" # textAlign
    textTransform = "text-transform" # textTransform

    boxShadow = "box-shadow" # boxShadow" # theme.shadows[value]
    displayPrint = "display" # displayPrint" # none
    display = "display" # displayRaw" # none
    alignContent = "align-content" # alignContent" # none
    alignItems = "align-items" # alignItems" # none
    alignSelf = "align-self" # alignSelf" # none
    flex = "flex" # flex" # none
    flexDirection = "flex-direction" # flexDirection" # none
    flexGrow = "flex-grow" # flexGrow" # none
    flexShrink = "flex-shrink" # flexShrink" # none
    flexWrap = "flex-wrap" # flexWrap" # none
    justifyContent = "justify-content" # justifyContent" # none
    order = "order" # order" # none
    opacity = "opacity" # order" # none
    gap = "gap" # gap" # theme.spacing(value)
    columnSpacing = "column-gap" # columnSpacing" # theme.spacing(value)
    rowSpacing = "row-gap" # rowSpacing" # theme.spacing(value)
    gridColumn = "grid-column" # gridColumn" # none
    gridRow = "grid-row" # gridRow" # none
    gridAutoFlow = "grid-auto-flow" # gridAutoFlow" # none
    gridAutoColumns = "grid-auto-columns" # gridAutoColumns" # none
    gridAutoRows = "grid-auto-rows" # gridAutoRows" # none
    gridTemplateColumns = "grid-template-columns" # gridTemplateColumns" # none
    gridTemplateRows = "grid-template-rows" # gridTemplateRows" # none
    gridTemplateAreas = "grid-template-areas" # gridTemplateAreas" # none
    gridArea = "grid-area" # gridArea" # none
    bgcolor = "background-color" # bgcolor" # theme.palette[value]
    bgImage = "background-image" # bgcolor" # theme.palette[value]
    backgroundImage = "background-image" # bgcolor" # theme.palette[value]
    color = "color" # color" # theme.palette[value]
    bottom = "bottom" # bottom" # none
    left = "left" # left" # none
    position = "position" # position" # none
    right = "right" # right" # none
    top = "top" # top" # none
    zIndex = "z-index" # zIndex" # theme.zIndex[value]
    height = "height" # height" # none
    maxHeight = "max-height" # maxHeight" # none
    maxWidth = "max-width" # maxWidth" # none
    minHeight = "min-height" # minHeight" # none
    minWidth = "min-width" # minWidth" # none
    width = "width" # width" # none
    boxSizing = "box-sizing" # boxSizing" # none

    subcontrolOrigin = 'subcontrol-origin'
    subcontrolPosition = 'subcontrol-osition'
    width = "width"
