from .icons import ICONS
from .favicon import FAVICON


class QTMUI_ASSETS:
    FAVICON = FAVICON
    ICONS = ICONS

"""
pyside6-rcc D:/mimui/qtmui/qtmui/qtmui/qtmui_assets/resources.qrc -o D:/mimui/qtmui/qtmui/qtmui/qtmui_assets/resources_rc.py
"""