class FAVICON:
    FAVICON=":/favicon/favicon/favicon.ico"
    FAVICON_512=":/favicon/favicon/android-chrome-512x512.png"
    FAVICON_192=":/favicon/favicon/android-chrome-192x192.png"
    FAVICON_APPLE=":/favicon/favicon/apple-touch-icon.png"
    FAVICON_16=":/favicon/favicon/favicon-16x16.png"
    FAVICON_32=":/favicon/favicon/favicon-32x32.png"
    FAVICON_QTMUI=":/favicon/favicon/favicon_qtmui.png"

"""
pyside6-rcc D:/mimui/qtmui/qtmui/qtmui/qtmui_assets/resources.qrc -o D:/mimui/qtmui/qtmui/qtmui/qtmui_assets/resources_rc.py
"""